<?php

declare(strict_types=1);

namespace Drupal\graphql_export\Commands;

use Consolidation\AnnotatedCommand\CommandData;
use Drupal\Core\Site\Settings;
use Drupal\graphql\Entity\Server;
use Drupal\graphql_export\GraphqlExportService;
use Drush\Commands\DrushCommands;

/**
 * Adds the post-command hook to the config:export command.
 */
class GraphqlExportCommands extends DrushCommands {

  /**
   * Construct a new export controller.
   *
   * @param \Drupal\graphql_export\GraphqlExportService $exportService
   *   The export service.
   */
  public function __construct(
    protected GraphqlExportService $exportService
  ) {}

  /**
   * Export the GraphQL Schemas.
   *
   * @hook post-command config:export
   */
  public function postCommand($result, CommandData $commandData): void {
    $settings = Settings::get('graphql_export', []);
    if (empty($settings)) {
      return;
    }

    $servers = Server::loadMultiple(array_keys($settings));

    foreach ($settings as $server_id => $setting) {
      $server = $servers[$server_id] ?? NULL;

      if (!$server) {
        $this->logger()->warning(dt('GraphQL Export: Server @id skipped, does not exist.', [
          '@id' => $server_id,
        ]));
        continue;
      }

      try {
        if (!empty($setting['graphqls'])) {
          $data = $this->exportService->printSchema($server);
          $filename = $this->writeFile($setting['graphqls'], $data, $server->id() . '.graphqls');

          $this->logger()->success(dt('GraphQL Export: Exported schema @id (@filename)', [
            '@id' => $server->id(),
            '@filename' => $filename,
          ]));
        }

        if (!empty($setting['json'])) {
          $data = $this->exportService->printJson($server);
          $filename = $this->writeFile($setting['json'], $data, $server->id() . '.json');

          $this->logger()->success(dt('GraphQL Export: Exported JSON @id (@filename)', [
            '@id' => $server->id(),
            '@filename' => $filename,
          ]));
        }
      }
      catch (\Throwable $e) {
        $this->logger()->error(dt('GraphQL Export: Failed with error @message', [
          '@message' => $e->getMessage(),
        ]));
      }
    }
  }

  /**
   * Write the file.
   *
   * @param string $path
   *   The path to write the file.
   * @param string $data
   *   The data to write.
   * @param string $default_filename
   *   The default filename if path is a directory.
   *
   * @return string
   *   The path to the file.
   *
   * @throws \RuntimeException
   */
  private function writeFile(string $path, string $data, string $default_filename): string {
    if (is_dir($path)) {
      $path = rtrim($path, DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR . $default_filename;
    }

    if (!file_put_contents($path, $data)) {
      throw new \RuntimeException(sprintf('Could not write GraphQL schema to %s', $path));
    }

    return $path;
  }

}
